--[[
More Blocks: stair definitions

Copyright (c) 2011-2017 Hugo Locurcio and contributors.
Licensed under the zlib license. See LICENSE.md for more information.
--]]

local S = moreblocks.intllib
local nodelight = ocutil.nodename_light_source

local enable_moreblocks_old_place =
    minetest.setting_getbool ("enable_moreblocks_old_place")

-- Node will be called <modname>:stair_<subname>

local stairs_defs = {
    [""] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.5, -0.5, 0.5, 0, 0.5},
                {-0.5, 0, 0, 0.5, 0.5, 0.5},
            },
        },
    },
    ["_half"] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.5, -0.5, 0, 0, 0.5},
                {-0.5, 0, 0, 0, 0.5, 0.5},
            },
        },
    },
    ["_right_half" ]= {
        node_box = {
            type = "fixed",
            fixed = {
                {0, -0.5, -0.5, 0.5, 0, 0.5},
                {0, 0, 0, 0.5, 0.5, 0.5},
            },
        },
    },
    ["_inner"] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.5, -0.5, 0.5, 0, 0.5},
                {-0.5, 0, 0, 0.5, 0.5, 0.5},
                {-0.5, 0, -0.5, 0, 0.5, 0},
            },
        },
    },
    ["_outer"] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.5, -0.5, 0.5, 0, 0.5},
                {-0.5, 0, 0, 0, 0.5, 0.5},
            },
        },
    },
    ["_alt"] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.5, -0.5, 0.5, 0, 0},
                {-0.5, 0, 0, 0.5, 0.5, 0.5},
            },
        },
    },
    ["_alt_1"] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.0625, -0.5, 0.5, 0, 0},
                {-0.5, 0.4375, 0, 0.5, 0.5, 0.5},
            },
        },
    },
    ["_alt_2"] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.125, -0.5, 0.5, 0, 0},
                {-0.5, 0.375, 0, 0.5, 0.5, 0.5},
            },
        },
    },
    ["_alt_4"] = {
        node_box = {
            type = "fixed",
            fixed = {
                {-0.5, -0.25, -0.5, 0.5, 0, 0},
                {-0.5, 0.25, 0, 0.5, 0.5, 0.5},
            },
        },
    },
}

for k,v in pairs(stairs_defs) do
    table.insert(stairsplus.shapes_list, { "stair_", k })
end

function stairsplus:register_stair_alias(modname_old, subname_old, modname_new, subname_new)
    local defs = stairsplus.copytable(stairs_defs)
    for alternate, def in pairs(defs) do
        minetest.register_alias(modname_old .. ":stair_" .. subname_old .. alternate, modname_new .. ":stair_" .. subname_new .. alternate)
    end
end

function stairsplus:register_stair_alias_force(modname_old, subname_old, modname_new, subname_new)
    local defs = stairsplus.copytable(stairs_defs)
    for alternate, def in pairs(defs) do
        minetest.register_alias_force(modname_old .. ":stair_" .. subname_old .. alternate, modname_new .. ":stair_" .. subname_new .. alternate)
    end
end

function stairsplus:register_stair (modname, subname, recipeitem, fields)
    if  fields.sounds == nil then
        fields.sounds =  default.sound_stone()
    end

    local defs = stairsplus.copytable(stairs_defs)
    local desc = S("%s Stairs"):format(fields.description)
    for alternate, def in pairs(defs) do
        for k, v in pairs(fields) do
            def[k] = v
        end

        def.drawtype   = "nodebox"
        def.paramtype  = "light"
        def.paramtype2 = def.paramtype2 or "facedir"

        if enable_moreblocks_old_place then
            def.on_place = minetest.rotate_node
        else
            def.on_place = function (itemstack, placer, pointed_thing)
                if pointed_thing.type ~= "node" then
                    return itemstack
                end

                local p0     = pointed_thing.under
                local p1     = pointed_thing.above
                local param2 = 0

                local placer_pos = placer:getpos()
                if placer_pos then
                    local dir = {
                        x = p1.x - placer_pos.x ,
                        y = p1.y - placer_pos.y ,
                        z = p1.z - placer_pos.z ,
                    }
                    param2 = minetest.dir_to_facedir (dir)
                end

                if p0.y - 1 == p1.y then
                    param2 = param2 + 20
                    if param2 == 21 then
                        param2 = 23
                    elseif param2 == 23 then
                        param2 = 21
                    end
                end

                return minetest.item_place (itemstack,
                    placer, pointed_thing, param2)
            end
        end

        def.description = desc
        def.groups = stairsplus:prepare_groups(fields.groups)
        if fields.drop and not (type(fields.drop) == "table") then
            def.drop = modname .. ":stair_" .. fields.drop .. alternate
        end

        if  def.light_source == nil then
            def.light_source =  nodelight (recipeitem)
        end

        minetest.register_node(":" .. modname .. ":stair_" .. subname .. alternate, def)
    end
--  minetest.register_alias("stairs:stair_" .. subname, modname .. ":stair_" .. subname)
    circular_saw.known_nodes[recipeitem] = {modname, subname}

    -- Some saw-less recipes:

-- RJK 2019: Slabs and stairs of "moreblocks:" nodes  belong to "more-
-- blocks:". Slabs and stairs of anything else belong to "stairs:"  as
-- long as they exist in "stairs:".

    local wantname = modname

    if ocutil.not_starts_with (recipeitem, "moreblocks:") then
        local maybe_node =
            "stairs:stair_" .. modname .. "_" .. subname

        if  minetest.registered_nodes [maybe_node] ~= nil then
            wantname = "stairs"
        end
    end

    if wantname ~= "stairs" then
        minetest.register_craft({
            output = wantname .. ":stair_" .. subname .. " 8",
            recipe = {
                {recipeitem, "", ""},
                {recipeitem, recipeitem, ""},
                {recipeitem, recipeitem, recipeitem},
            },
        })

        minetest.register_craft({
            output = wantname .. ":stair_" .. subname .. " 8",
            recipe = {
                {"", "", recipeitem},
                {"", recipeitem, recipeitem},
                {recipeitem, recipeitem, recipeitem},
            },
        })
    end

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname,
        recipe = {modname .. ":panel_" .. subname, modname .. ":slab_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname,
        recipe = {modname .. ":panel_" .. subname, modname .. ":panel_" .. subname, modname .. ":panel_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname .. "_outer",
        recipe = {modname .. ":micro_" .. subname, modname .. ":slab_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname .. "_half",
        recipe = {modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname .. "_half",
        recipe = {modname .. ":panel_" .. subname, modname .. ":micro_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname .. "_right_half",
        recipe = {modname .. ":stair_" .. subname .. "_half"},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname,
        recipe = {modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname .. "_inner",
        recipe = {modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname .. "_outer",
        recipe = {modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname, modname .. ":micro_" .. subname},
    })

    minetest.register_craft({
        type = "shapeless",
        output = modname .. ":stair_" .. subname,
        recipe = {modname .. ":panel_" .. subname, modname .. ":panel_" .. subname, modname .. ":panel_" .. subname},
    })

    minetest.register_craft({ -- See mirrored variation of the recipe below.
        output = modname .. ":stair_" .. subname .. "_alt",
        recipe = {
            {modname .. ":panel_" .. subname, ""},
            {""                            , modname .. ":panel_" .. subname},
        },
    })

    minetest.register_craft({ -- Mirrored variation of the recipe above.
        output = modname .. ":stair_" .. subname .. "_alt",
        recipe = {
            {""                            , modname .. ":panel_" .. subname},
            {modname .. ":panel_" .. subname, ""},
        },
    })
end
