-- File:     codersea/tidyup.lua
-- Purpose:  Some of CoderSea's node and entity tidy-up code
-- Credits:  See "codersea.md"
-- Licenses: See "LICENSE"
-- Revision: 200329

-- ===================================================================

local map_dirt = {
    "clams:dirtalgae"                      ,
    "clams:dirtalgaeused"                  ,
    "clams:collectedalgae"                 ,
    "clams:crushedwhite"                   ,

    "codersea:dirtkelp"                    ,
    "codersea:dirtseagrass"                ,
    "codersea:seaplantsdirtkelpbrown"      ,
    "codersea:seaplantsdirtkelpgreen"      ,
    "codersea:seaplantsdirtseagrassgreen"  ,
    "codersea:seaplantsdirtseagrassred"    ,

    "seaplants:seaplantsdirtkelpbrown"     ,
    "seaplants:seaplantsdirtkelpgreen"     ,
    "seaplants:seaplantsdirtseagrassgreen" ,
    "seaplants:seaplantsdirtseagrassred"   ,
}

for _,node_name in ipairs (map_dirt) do
    minetest.register_node (":" .. node_name, {
        groups = { map_dirt=1, not_in_creative_inventory=1 } ,
    })
end

minetest.register_lbm ({
    name              = "codersea:map_dirt"  ,
    nodenames         =  { "group:map_dirt" } ,
    run_at_every_load = true ,
    action            = function (pos, node)
        minetest.add_node (pos, { name = "default:dirt" })
    end ,
})

-- ===================================================================

local map_desert_sand = {
    "whiteshell:whiteshell" ,
}

for _,node_name in ipairs (map_desert_sand) do
    minetest.register_node (":" .. node_name, {
        groups = { map_desert_sand=1, not_in_creative_inventory=1 } ,
    })
end

minetest.register_lbm ({
    name              = "codersea:map_desert_sand" ,
    nodenames         = {  "group:map_desert_sand" } ,
    run_at_every_load = true ,
    action            = function (pos, node)
        minetest.add_node (pos, { name = "default:desert_sand" })
    end ,
})

-- ===================================================================

local map_sand = {
    "clams:sandalgae"                      ,
    "clams:sandalgaeused"                  ,
    "clams:collectedalgae"                 ,
    "clams:crushedwhite"                   ,

    "codersea:sandkelp"                    ,
    "codersea:sandseagrass"                ,
    "codersea:seaplantssandkelpbrown"      ,
    "codersea:seaplantssandkelpgreen"      ,
    "codersea:seaplantssandseagrassgreen"  ,
    "codersea:seaplantssandseagrassred"    ,

    "seaplants:seaplantssandkelpbrown"     ,
    "seaplants:seaplantssandkelpgreen"     ,
    "seaplants:seaplantssandseagrassgreen" ,
    "seaplants:seaplantssandseagrassred"   ,
}

for _,node_name in ipairs (map_sand) do
    minetest.register_node (":" .. node_name, {
        groups = { map_sand=1, not_in_creative_inventory=1 } ,
    })
end

minetest.register_lbm ({
    name              = "codersea:map_sand" ,
    nodenames         =  { "group:map_sand" } ,
    run_at_every_load = true ,
    action            = function (pos, node)
        minetest.add_node (pos, { name = "default:sand" })
    end ,
})

-- ===================================================================

local map_water_flowing = {
    "noairblocks:water_flowingx",
}

for _,node_name in ipairs (map_water_flowing) do
    minetest.register_node (":" .. node_name, {
        groups = { map_water_flowing=1,
not_in_creative_inventory=1 } ,
    })
end

minetest.register_lbm ({
    name              = "codersea:map_water_flowing" ,
    nodenames         =  { "group:map_water_flowing" } ,
    run_at_every_load = true ,
    action            = function (pos, node)
        minetest.add_node (pos, { name = "default:water_flowing" })
    end ,
})


-- ===================================================================

local map_water_source = {
    "seaplants:kelpgreen"        ,
    "seaplants:kelpgreenmiddle"  ,
    "seaplants:kelpbrown"        ,
    "seaplants:kelpbrownmiddle"  ,
--  "seaplants:seagrassgreen"    ,
--  "seaplants:seagrassred"      ,
    "noairblocks:water_sourcex"  ,
}

for _,node_name in ipairs (map_water_source) do
    minetest.register_node (":" .. node_name, {
        groups = { map_water_source=1,
not_in_creative_inventory=1 } ,
    })
end

minetest.register_lbm ({
    name              = "codersea:map_water_source" ,
    nodenames         =  { "group:map_water_source" } ,
    run_at_every_load = true ,
    action            = function (pos, node)
        minetest.add_node (pos, { name = "default:water_source" })
    end ,
})

-- ===================================================================

local old_entities = {
    "clams:whiteshell"         ,
    "codersea:oddfish"         ,
}

local function delete_old_entity    (entity_name)
    minetest.register_entity (":" .. entity_name, {
        on_activate = function (self, staticdata)
            self.object:remove()
        end ,
    })
end

-- ===================================================================

for _,entity_name in ipairs (old_entities) do
    delete_old_entity (entity_name)
end

-- ===================================================================

if not codersea_disable_mobs and not codersea_reset then

    if not codersea_disable_dolidro    then
        delete_old_entity ("nssm:dolidrosaurus"       )
    end
    if not codersea_disable_fish       then
        delete_old_entity ("mobs_fish:clownfish"      )
        delete_old_entity ("mobs_fish:tropical"       )
    end
    if not codersea_disable_jellyfish  then
        delete_old_entity ("mobs_jellyfish:jellyfish" )
    end
    if not codersea_disable_octopus    then
        delete_old_entity ("nssm:octopus"             )
    end
    if not codersea_disable_shark      then
        delete_old_entity ("mobs_sharks:shark_lg"     )
        delete_old_entity ("mobs_sharks:shark_md"     )
        delete_old_entity ("mobs_sharks:shark_sm"     )
    end
    if not codersea_disable_turtle     then
        delete_old_entity ("mobs_turtles:turtle"      )
        delete_old_entity ("mobs_turtles:seaturtle"   )
    end
end

-- ===================================================================

if codersea_disable_dolidro    then
    delete_old_entity ("codersea:dolidrosaurus"   )
end
if codersea_disable_fish       then
    delete_old_entity ("codersea:clownfish"       )
    delete_old_entity ("codersea:tropical"        )
end
if codersea_disable_jellyfish  then
    delete_old_entity ("codersea:jellyfish"       )
end
if codersea_disable_octopus    then
    delete_old_entity ("codersea:octopus"         )
end
if codersea_disable_shark      then
    delete_old_entity ("codersea:smshark"         )
    delete_old_entity ("codersea:mdshark"         )
    delete_old_entity ("codersea:lgshark"         )
end
if codersea_disable_turtle     then
    delete_old_entity ("codersea:seaturtle"       )
end

-- ===================================================================

if codersea_disable_mobs or codersea_reset then
    delete_old_entity ("codersea:clownfish"     )
    delete_old_entity ("codersea:dolidrosaurus" )
    delete_old_entity ("codersea:jellyfish"     )
    delete_old_entity ("codersea:octopus"       )
    delete_old_entity ("codersea:seaturtle"     )
    delete_old_entity ("codersea:shark"         )
    delete_old_entity ("codersea:tropical"      )

    codersea_delete_mobs = true
end

-- ===================================================================

minetest.log ("action", "[codersea] tidyup.lua loaded")
