-- Badger. Descended from "dmobs" version.

-- ===================================================================

local lcname     = "badger"
local ucname     = "Badger"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  40000       ,

    day_mode     = true         ,
    min_light    =      0       ,
    max_light    =     20       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt"                  ,
        "default:dirt_with_dry_grass"   ,
        "default:dirt_with_grass"       ,
        "earthgen:dirt_with_dry_grass"  ,
        "ethereal:green_dirt"           ,
        "ethereal:green_dirt_top"       ,
        "fargen:dirt_with_leafy_grass"  ,
        "mg:dirt_with_dry_grass"        ,
        "mypaths:grass"                 ,
        "noisegrid:grass"               ,
    } ,

    add_egg   = true                    ,
    egg_image = "default_obsidian.png"  ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type                 = mobs_param.spawn_type                ,
    makes_footstep_sound = true                                 ,

    -- Behavior/sight/battle
    armor         = 130         ,
    attack_type   = "dogfight"  ,
    damage        =   2         ,
    hp_min        =  12         ,
    hp_max        =  22         ,
    passive       = false       ,
    reach         =   1         ,
    runaway       = true        ,
    view_range    =  14         ,

    -- Velocities
    run_velocity  =   1         ,
    walk_velocity =   0.7       ,

    -- Step/jump/fall parameters
    fall_damage   =  1          ,
    fall_speed    = -8          ,
    fear_height   =  4          ,
    jump          = false       ,
    jump_height   =  0          ,
    stepheight    =  1.2        ,

    -- Environment damage
    water_damage  =  1          ,
    lava_damage   =  2          ,
    light_damage  =  0          ,

    -- Misc. parameters
    floats        =  0          ,

    blood_texture        = "mobs_blood.png"                     ,
    collisionbox         = { -0.3, -0.15, -0.3, 0.3, 0.4, 0.3 } ,
    mesh                 = msname .. ".b3d"                     ,
    textures             = {{ msname .. ".jpg" }}               ,
    visual               = "mesh"                               ,
    visual_size          = { x=2, y=2 }                         ,

    drops = {
        { name   = "mobs:meat_raw",
          chance = 1, min = 1, max = 1
        } ,
    } ,

    specific_attack = codermobs.make_mob_list ({
        "chicken",
    }) ,

    replace_rate = 1,
    replace_what = {
        {"chicken_egg", "air", 0},
    },

    follow = { "mobs:meat_raw" } ,

    animation = {
        speed_normal = 12 ,
        speed_run    = 18 ,
        walk_start   = 34 ,
        walk_end     = 58 ,
        stand_start  =  1 ,
        stand_end    = 30 ,
        run_start    = 34 ,
        run_end      = 58 ,
        punch_start  = 60 ,
        punch_end    = 80 ,
    } ,

    on_rightclick = function (self, clicker)
        if mobs:feed_tame (self, clicker, 8, true, true) then
            return
        end

        mobs:capture_mob (self, clicker, 0, 5, 50, false, nil)
    end ,
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
