-- Birds. Descended from Sapier version.

-- ===================================================================

local codermobs_disable_monsters       =
    minetest.setting_getbool ("codermobs_disable_monsters"      )

local codermobs_logless                =
    minetest.setting_getbool ("codermobs_logless"               )

local codermobs_monster_global         =
    minetest.setting_getbool ("codermobs_monster_global"        )

local codermobs_peaceful_global        =
    minetest.setting_getbool ("codermobs_peaceful_global"       )

local codermobs_enable_thismob         =
    codermobs.mob_enabled ("bird")

-- ===================================================================

local minheight    = -31000
local maxheight    =  11000

-- ===================================================================

    local l_skins_gull = {
        { "codermobs_gull_white.png"    } ,
        { "codermobs_gull_black.png"    } ,
        { "codermobs_gull_gray.png"     } ,
        { "codermobs_gull_bluegray.png" } ,
    }

    local l_skins_bird = {
        { "codermobs_bird_blue.png"     } ,
        { "codermobs_bird_brown.png"    } ,
        { "codermobs_bird_gray.png"     } ,
        { "codermobs_bird_bluegray.png" } ,
        { "codermobs_bird_red.png"      } ,
        { "codermobs_bird_redish.png"   } ,
    }

    local l_anims = {
        speed_normal = 24 , speed_run = 24 ,
        stand_start  =  1 , stand_end = 95 ,
        walk_start   =  1 , walk_end  = 95 ,
        run_start    =  1 , run_end   = 95 ,
    }

    local l_gull_model          = "codermobs_gull.b3d"
    local l_thrush_model        = "codermobs_thrush.b3d"
    local l_egg_texture         = "default_cloud.png"
    local l_capture_chance_h    = 5
    local l_capture_chance_n    = 60
    local l_spawn_in            = { "air" }

    local l_spawn_near_gull     = {
        "default:leaves"        ,
        "default:pine_needles"  ,
        "default:jungleleaves"  ,
        "default:cactus"        ,
        "default:water_source"  ,
        "default:water_flowing" ,
    }

    local l_spawn_near_bird     = {
        "default:leaves"        ,
        "default:pine_needles"  ,
        "default:jungleleaves"  ,
        "default:cactus"        ,
        "default:water_source"  ,
        "default:water_flowing" ,
    }

local l_spawn_chance_gull = 30000
local l_spawn_chance_bird = 30000

ENABLE_GULLS       = true
ENABLE_LARGE_BIRDS = true
ENABLE_SMALL_BIRDS = true

if ENABLE_GULLS then
mobs:register_mob ("codermobs:gull", {
    type          = "animal"     ,
    passive       = true         ,
    hp_min        =   5          ,
    hp_max        =  10          ,
    armor         = 100          ,
    collisionbox  = { -1, -0.3, -1, 1, 0.3, 1 } ,

    visual        = "mesh"       ,
    mesh          = l_gull_model ,
    textures      = l_skins_gull ,
    rotate        = 270          ,
    walk_velocity =   4          ,
    run_velocity  =   6          ,
    fall_speed    =   0          ,
    stepheight    =   3          ,
    fly           = true         ,
    water_damage  =   0          ,
    lava_damage   =  10          ,
    light_damage  =   0          ,
    view_range    =  14          ,
    animation     = l_anims      ,

    on_rightclick = function(self, clicker)
        mobs:capture_mob (self, clicker,
            l_capture_chance_h, l_capture_chance_n, 0, true, nil)
    end
})

-- name, nodes, neighbors, min_light, max_light, interval, chance, active_object_count, min_height, max_height

if codermobs_enable_thismob then
mobs:spawn_specific ("codermobs:gull", l_spawn_in,
{ "default:leaves", "default:pine_needles", "default:dirt_with_dry_grass",
"default:cactus" },
5, 20, 30, 12000, -1, minheight, maxheight)
end

if codermobs_enable_thismob then
mobs:spawn_specific ("codermobs:gull", l_spawn_in,
{ "default:water_source", "default:water_flowing" },
5, 20, 30, 24000, -1, minheight, maxheight)
end

mobs:register_egg ("codermobs:gull", "Gull", l_egg_texture, 1)
end

if ENABLE_LARGE_BIRDS then
mobs:register_mob ("codermobs:lgbird", {
    type = "animal",
    passive = true,
    hp_min = 5,
    hp_max = 10,
    armor = 100,
    collisionbox = {-0.5, -0.3, -0.5, 0.5, 0.3, 0.5},
    visual = "mesh",
    mesh = l_thrush_model,
    textures = l_skins_bird,
    visual_size = {x=.5, y=.5},
    rotate = 270,
    walk_velocity = 4,
    run_velocity = 6,
    fall_speed = 0,
    stepheight = 3,
    fly = true,
    water_damage = 0,
    lava_damage = 10,
    light_damage = 0,
    view_range = 12,
    animation = l_anims,

    on_rightclick = function(self, clicker)
        mobs:capture_mob(self, clicker, l_capture_chance_h, l_capture_chance_n, 0, true, nil)
    end
})

-- name, nodes, neighbors, min_light, max_light, interval, chance, active_object_count, min_height, max_height

if codermobs_enable_thismob then
mobs:spawn_specific ("codermobs:lgbird", l_spawn_in,
{
    "default:leaves"       ,
    "default:pine_needles" ,
    "default:dirt_with_dry_grass" ,
    "default:cactus"
} ,
5, 20, 30, 12000, -1, minheight, maxheight)
end

if codermobs_enable_thismob then
mobs:spawn_specific ("codermobs:lgbird", l_spawn_in,
{ "default:water_source", "default:water_flowing" },
5, 20, 30, 24000, -1, minheight, maxheight)
end

mobs:register_egg ("codermobs:lgbird", "Large bird", l_egg_texture, 1)
end

if ENABLE_SMALL_BIRDS then
mobs:register_mob ("codermobs:smbird", {
    type = "animal",
    passive = true,
    hp_min = 2,
    hp_max = 5,
    armor = 100,
    collisionbox = {-0.25, -0.3, -0.25, 0.25, 0.3, 0.25},
    visual = "mesh",
    mesh = l_thrush_model,
    textures = l_skins_bird,
    visual_size = {x=.25, y=.25},
    rotate = 270,
    walk_velocity = 4,
    run_velocity = 6,
    fall_speed = 0,
    stepheight = 3,
    fly = true,
    water_damage = 0,
    lava_damage = 10,
    light_damage = 0,
    view_range = 10,
    animation = l_anims,
    on_rightclick = function(self, clicker)
        mobs:capture_mob(self, clicker, l_capture_chance_h, l_capture_chance_n, 0, true, nil)
    end
})

--name, nodes, neighbors, min_light, max_light, interval, chance, active_object_count, min_height, max_height

if codermobs_enable_thismob then
mobs:spawn_specific ("codermobs:smbird", l_spawn_in,
{ "default:leaves", "default:pine_needles", "default:dirt_with_dry_grass",
"default:cactus" },
5, 20, 30, 12000, -1, minheight, maxheight)
end

if codermobs_enable_thismob then
mobs:spawn_specific ("codermobs:smbird", l_spawn_in,
{ "default:water_source", "default:water_flowing" },
5, 20, 30, 24000, -1, minheight, maxheight)
end

mobs:register_egg ("codermobs:smbird", "Small bird", l_egg_texture, 1)
end
