-- Lava Flan.

-- ===================================================================

local lcname     = "lava_flan"
local ucname     = "Lava Flan"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =   2500       ,

    day_mode     = true         ,
    min_light    =      0       ,
    max_light    =     30       ,
    min_height   = -31000       ,
    max_height   =     75       ,

    spawn_type   = "monster"    ,

    spawn_nodes  = {
        "default:lava_source"       ,
    } ,

    add_egg   = true                ,
    egg_image = "default_lava.png"  ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,

    armor           = 80                ,
    attack_type     = "dogfight"        ,
    damage          =  3                ,
    floats          =  1                ,
    hp_min          = 10                ,
    hp_max          = 35                ,
    stepheight      = 1.2               ,
    jump_height     =   0               ,
    jump            = false             ,
    lava_damage     =  0                ,
    light_damage    =  0                ,
    passive         = false             ,
    reach           =  2                ,
    run_velocity    =  2                ,
    view_range      = 10                ,
    walk_velocity   =  0.5              ,
    water_damage    =  5                ,

    blood_texture   = "fire_basic_flame.png"              ,
    collisionbox    = { -0.5, -0.5, -0.5, 0.5, 1.5, 0.5 } ,
    visual          = "mesh"                              ,
    mesh            = "codermobs_lava_flan.x"             ,

    drops = {
        { name="mobs:lava_orb", chance=15, min=1, max=1 } ,
    } ,

    sounds = {
        random  = "codermobs_lava_flan" ,
        war_cry = "codermobs_lava_flan" ,
    } ,

    textures = {
        { "codermobs_lava_flan1.png" } ,
        { "codermobs_lava_flan2.png" } ,
        { "codermobs_lava_flan3.png" } ,
    },

    animation = {
        speed_normal = 15 ,
        speed_run    = 15 ,
        stand_start  =  0 ,
        stand_end    =  8 ,
        walk_start   = 10 ,
        walk_end     = 18 ,
        run_start    = 20 ,
        run_end      = 28 ,
        punch_start  = 20 ,
        punch_end    = 28 ,
    },

    makes_footstep_sound = false ,

    on_die = function (self, pos)
        if  minetest.get_node (pos).name == "air" then
            minetest.set_node (pos, { name = "fire:basic_flame" })
        end

        self.object:remove()

        minetest.add_particlespawner ({
            amount = 20,
            time   = 0.25,
            minpos = pos,
            maxpos = pos,
            minvel = { x = -2, y =  -2, z = -2 } ,
            maxvel = { x =  2, y =   2, z =  2 } ,
            minacc = { x =  0, y = -10, z =  0 } ,
            maxacc = { x =  0, y = -10, z =  0 } ,
            minexptime = 0.1,
            maxexptime = 1,
            minsize = 1.0,
            maxsize = 2.0,
            texture = "fire_basic_flame.png",
        })
    end,
}

-- ===================================================================

-- lava orb
minetest.register_craftitem (":mobs:lava_orb", {
    description     = "Lava orb"               ,
    inventory_image = "codermobs_lava_orb.png" ,
})

minetest.register_craft ({
    type     = "fuel"           ,
    recipe   = "mobs:lava_orb"  ,
    burntime = 80               ,
})

-- ===================================================================

-- Lava Pick (digs and smelts at same time)
local old_handle_node_drops = minetest.handle_node_drops

function minetest.handle_node_drops (pos, drops, digger)
    if digger == nil then return end

    -- are we holding Lava Pick?
    if digger:get_wielded_item():get_name() ~= "mobs:pick_lava" then
        return old_handle_node_drops (pos, drops, digger)
    end

    -- reset new smelted drops
    local hot_drops = {}

    -- loop through current node drops
    for _, drop in pairs (drops) do

        -- get cooked output of current drops
        local stack = ItemStack (drop)
        local output = minetest.get_craft_result ({
            method = "cooking" ,
            width  = 1         ,
            items  = { drop } ,
        })

        -- if we have cooked result then add to new list
        if output
        and output.item
        and not output.item:is_empty() then

            table.insert (hot_drops,
                ItemStack ({
                    name  = output.item:get_name() ,
                    count = output.item:to_table().count ,
                })
            )
        else -- if not then return normal drops
            table.insert (hot_drops, stack)
        end
    end

    return old_handle_node_drops (pos, hot_drops, digger)
end

-- ===================================================================

minetest.register_tool (":mobs:pick_lava", {
    description     = "Lava Pickaxe"       ,
    inventory_image = "mobs_pick_lava.png" ,

    tool_capabilities = {
        full_punch_interval = 0.4,
        max_drop_level = 3 ,
        groupcaps = {
            cracky = {
                times= {
                    [1]=1.80 , [2]=0.80 , [3]=0.40} ,
                    uses=40, maxlevel=3 ,
                } ,
        } ,
        damage_groups = { fleshy=6 } ,
    } ,
})

-- ===================================================================

minetest.register_craft ({
    output = "mobs:pick_lava",

    recipe = {
{ "mobs:lava_orb" , "mobs:lava_orb"          , "mobs:lava_orb" } ,
{ ""              , "default:obsidian_shard" , ""              } ,
{ ""              , "default:obsidian_shard" , ""              } ,
    }
})

-- ===================================================================

codermobs.setup_mob()
mobs:alias_mob ("mobs_monster:lava_flan" , obj_name)
codermobs.log_done()

-- ===================================================================
-- End of file.
