-- MultiCraft Pig.

-- ===================================================================

local lcname     = "mcpig"
local ucname     = "MC Pig"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  40000       ,

    day_mode     = true         ,
    min_light    =      5       ,
    max_light    =     20       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt"                  ,
        "default:sand"                  ,
        "default:redsand"               ,
        "default:snow"                  ,
        "default:snowblock"             ,
        "default:dirt_with_snow"        ,
        "default:dirt_with_grass"       ,
        "default:dirt_with_dry_grass"   ,
        "noisegrid:grass"               ,
    } ,

    add_egg   = false                   ,
    egg_image = msname .. "_egg.png"    ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    makes_footstep_sound = true  ,

-- ===================================================================
-- Basic parameters.

    -- Most behavior/sight/battle
    armor          = 200         ,
    attack_odds    =  85         ,
    damage         =   1         ,
    group_attack   = true        ,
    hp_max         =  15         ,
    hp_min         =   5         ,
    passive        = false       ,
    runaway        = true        ,
    view_range     =  10         ,

    -- Velocities
    run_velocity   =   3         ,
    walk_velocity  =   2         ,

    -- Step/jump/fall parameters
    fear_height    =   2         ,
    jump_height    =   0         ,
    jump           = false       ,
    stepheight     =   1.2       ,

    -- Environment damage
    lava_damage    =   5         ,
    water_damage   =   1         ,

    -- Misc. parameters
    floats         =   0         ,

-- ===================================================================
-- Visual components.

    collisionbox     = { -0.50, -0.01, -0.50, 0.50, 1.10, 0.50 } ,
    mesh             = msname .. ".b3d"                          ,
    textures         = {{ msname .. "_mesh.png" }}               ,
    visual           = "mesh"                                    ,
    visual_size      = { x = 1, y = 1 }                          ,

    animation        = {
        speed_normal =  20 ,
        stand_start  =   0 ,
        stand_end    =  60 ,
        walk_start   =  61 ,
        walk_end     =  80 ,
        punch_start  =  90 ,
        punch_end    = 110 ,
    } ,

-- ===================================================================
-- Other components.

    drops = {
        { name="mobs:meat_raw", chance=1, min=1, max=1 } ,
    } ,

    follow           = {
        "default:apple"        ,
        "farming:potato"       ,
    } ,

    sounds           = {
        random = msname             ,
        attack = msname .. "_angry" ,
    } ,

    on_rightclick    = function (self, clicker)
        mobs:feed_tame (self, clicker, 8, true, true)
    end ,
}

-- ===================================================================

codermobs.setup_mob()
mobs:alias_mob ("codermobs:pig"      , obj_name)
codermobs.log_done()

-- ===================================================================
-- End of file.
