-- Zebra. Descended from Farlands version.

-- ===================================================================

local lcname     = "zebra"
local ucname     = "Zebra"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance = 120000       ,

    day_mode     = true         ,
    min_light    =      7       ,
    max_light    =     20       ,
    min_height   = -31000       ,
    max_height   =  11000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_dry_grass"   ,
    } ,

    add_egg   = true                    ,
    egg_image = "default_obsidian.png"  ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    makes_footstep_sound =  true ,

    -- Most behavior/sight/battle
    armor          = 120         ,
    attack_type    = "dogfight"  ,
    damage         =   2         ,
    hp_max         =  60         ,
    hp_min         =  50         ,
    passive        = false       ,
    reach          =   1         ,
    runaway        = true        ,
    view_range     =  14         ,

    -- Velocities
    run_velocity   =   3.2       ,
    walk_velocity  =   1.5       ,

    -- Step/jump/fall parameters
    jump           = false       ,
    jump_height    =   0         ,
    fall_damage    =   1         ,
    fear_height    =   2         ,
    stepheight     =   2.2       ,

    -- Environment damage
    lava_damage    =   2         ,
    light_damage   =   0         ,
    water_damage   =   1         ,

    -- Misc. parameters
    floats         =   0         ,

    blood_texture    = "mobs_blood.png"                          ,
    collisionbox     = { -0.50, 0.00, -0.50, 0.50, 1.50, 0.50 }  ,
    mesh             = "codermobs_farhorse.b3d"                  ,
    textures         = {{ msname .. ".png" }}                    ,
    visual           = "mesh"                                    ,

    animation        = {
        walk_start   =  25 ,
        walk_end     =  45 ,
        run_start    =  50 ,
        run_end      =  70 ,
        stand_start  =  80 ,
        stand_end    = 120 ,
        speed_normal =  16 ,
        speed_run    =  22 ,
    } ,

    drops            = {
        { name="mobs:meat_raw" , chance=1 , min=1, max=3 } ,
        { name="mobs:leather"  , chance=1 , min=1, max=2 } ,
    } ,

    follow           = { "farming:wheat" }    ,

    replace_rate     = 10                     ,
    replace_what     = {
        "default:grass_3" , "default:grass_4" ,
        "default:grass_5" ,
    } ,
    replace_with  = "air"                     ,

    specific_attack = codermobs.make_mob_list ({
        "snake_garter", "snake_ice",
    }) ,

    on_rightclick = function (self, clicker)
        if mobs:feed_tame (self, clicker, 8, true, true) then
            return
        end

        if self.driver and clicker == self.driver then
        object_detach(self, clicker, {x=1, y=0, z=1})
        elseif not self.driver and clicker:get_wielded_item():get_name() == "mobs:saddle" then
-- RJK:
--      object_attach(self, clicker, {x=0, y=25, z=0}, true, {x=2, y=6, z=0})
        object_attach(self, clicker, {x=0, y=25, z=0}, {x=2, y=6, z=0})
        end
        mobs:capture_mob (self, clicker, 0, 5, 50, false, nil)
    end ,

    do_custom = function (self, dtime)
        if self.driver then
            self.state = "stand"
            object_drive(self, dtime, {
                speed = 6.3,
                decell = 0.65,
                moving_anim = {x=50, y=70},
                stand_anim = {x=1, y=1},
                ignore_water = true
            })
            return false
        end
        return true
    end ,
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
