-- ===================================================================
-- Mod main table.

upper_realms = {}

-- ===================================================================
-- Bucket Game Upper Realms.

--[[
    Height           Realm        Outer Space?
    ==============   ==========   ============
    below to  2000   Earth        No      In non-Moontest mode
    below to  2000   Moon         Yes     In Moontest     mode
     5000 to  6000   BRON         No
    18000 to 18992   Planetoids   No
    19120 to 20992   MT Space     Yes     Initially just vacuum
    21008 to 22000   Asteroids    Yes
    26992 to 30000   Earth 2      No      In non-Moontest  mode
    26992 to 30000   Earth        No      In Moontest      mode

----------------------------------------------------------------------

There  should  be a  vertical separation  of  at least a mapchunk  (96
nodes) between most realms.

The  MT Space-Asteroids pair of realms is an exception. They're separ-
ated by a layer that's just a mapblock (16 nodes)  tall.  The  limited
separation is believed to be safe in this case.

----------------------------------------------------------------------

If "loud_walking" is installed, the associated realm occupies a corner
of MT Space.  Note: The corner is more than a mapchunk below the start
of the Asteroids realm.

----------------------------------------------------------------------

MT Space  plus  the Asteroids  realm  make up, together,  a block that
we'll refer to, for now, as Space-1.

Space-1 includes the  thin layer, mentioned above,  which separates MT
Space and Asteroids.
--]]

-- ===================================================================
-- Boundaries of primary upper realms.

-- The numbers used here should all be integer multiples of 16.

upper_realms.moon              = {}
upper_realms.moon.y_min        = nil
upper_realms.moon.y_max        =  2000

upper_realms.planetoids        = {}
upper_realms.planetoids.y_min  = 18000
upper_realms.planetoids.y_max  = 18992

upper_realms.mtspace           = {}
upper_realms.mtspace.y_min     = 19120
upper_realms.mtspace.y_max     = 20992

if ocutil.bool_setting ("redcat_world") then
upper_realms.mtspace.y_min     =  4000
upper_realms.mtspace.y_max     = 20992
end

upper_realms.asteroids         = {}
upper_realms.asteroids.y_min   = 21008
upper_realms.asteroids.y_max   = 22000

upper_realms.space1            = {}
upper_realms.space1.y_min      = upper_realms.mtspace.y_min
upper_realms.space1.y_max      = upper_realms.asteroids.y_max

-- ===================================================================
-- Boundaries of "Loud Walking" realm.

upper_realms.loud_walking      = {}

upper_realms.loud_walking.minp = {
    x = -30400 ,
    y =  upper_realms.mtspace.y_min  + 500 ,
    z = -30400 ,
}

upper_realms.loud_walking.baseline =
    upper_realms.loud_walking.minp.y + 300

upper_realms.loud_walking.maxp = {
    x = upper_realms.loud_walking.minp.x   + 6656 ,
    y = upper_realms.loud_walking.baseline +  300 ,
    z = upper_realms.loud_walking.minp.z   + 6656 ,
}

-- ===================================================================

local enable_moontest =
    minetest.setting_getbool ("enable_moon"     ) or
    minetest.setting_getbool ("enable_moontest" )

-- ===================================================================

upper_realms.is_outer_space = function (y)
    local is_space = false

    if  enable_moontest and
        y <= upper_realms.moon.y_max then
        is_space = true
    end

    if  y >= upper_realms.space1.y_min and
        y <= upper_realms.space1.y_max then
        is_space = true
    end

    return is_space
end

-- ===================================================================

upper_realms.not_outer_space = function (y)
    if upper_realms.is_outer_space (y) then return false end
    return true
end

-- ===================================================================

upper_realms.is_loud_walking = function (minp, maxp)

    if ocutil.mod_missing ("loud_walking") then return false end
    if maxp == nil then maxp = minp end

    local lw_minp = upper_realms.loud_walking.minp
    local lw_maxp = upper_realms.loud_walking.maxp

    if  (minp.x >= lw_minp.x) and (maxp.x <= lw_maxp.x) and
        (minp.y >= lw_minp.y) and (maxp.y <= lw_maxp.y) and
        (minp.z >= lw_minp.z) and (maxp.z <= lw_maxp.z) then
        return true
    end

    return false
end

-- ===================================================================

upper_realms.not_loud_walking = function (minp, maxp)
    if upper_realms.is_loud_walking (minp, maxp) then return false end
    return true
end

-- ===================================================================

upper_realms.set_space_light = function (vm, area, minp, maxp, light)

    if  light == nil then
        light =  default.space_light_generic
    end

    local ltdata = vm:get_light_data()
    local chg    = false

-- The "+ 1" used in the following code block is a  work-around for an
-- apparent bug in the MT mapgen API.

    for ii in area:iterp (minp, maxp) do
        local sun = ltdata [ii] % 0x10
        local nat = ltdata [ii] - sun
        if    nat       < ((light + 0) * 0x10) then
            ltdata [ii] = ((light + 1) * 0x10) + sun
            chg = true
        end
    end

    if chg then
        vm:set_light_data (ltdata)

-- This step is presently skipped:
--      vm:update_liquids()

        vm:write_to_map (false)
    end
end

-- ===================================================================
-- End of file.
